@@ -134,6 +134,9 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         request = {'type': ovn_const.REQ_TYPE_POOL_CREATE,
                    'info': request_info}
         self._ovn_helper.add_request(request)
+        if pool.healthmonitor is not None and not isinstance(
+                pool.healthmonitor, o_datamodels.UnsetType):
+            self.health_monitor_create(pool.healthmonitor)
 
     def pool_delete(self, pool):
         for member in pool.members:
@@ -238,11 +241,15 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         admin_state_up = member.admin_state_up
         subnet_id = member.subnet_id
         if (isinstance(subnet_id, o_datamodels.UnsetType) or not subnet_id):
-            subnet_id = self._ovn_helper._get_subnet_from_pool(member.pool_id)
-            if not subnet_id:
+            subnet_id, subnet_cidr = self._ovn_helper._get_subnet_from_pool(
+                member.pool_id)
+            if not (subnet_id and
+                    self._ovn_helper._check_ip_in_subnet(member.address,
+                                                         subnet_cidr)):
                 msg = _('Subnet is required, or Loadbalancer associated with '
                         'Pool must have a subnet, for Member creation '
-                        'with OVN Provider Driver')
+                        'with OVN Provider Driver if it is not the same as '
+                        'LB VIP subnet')
                 raise driver_exceptions.UnsupportedOptionError(
                     user_fault_string=msg,
                     operator_fault_string=msg)
@@ -272,11 +279,30 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         self._ovn_helper.add_request(request)
 
     def member_delete(self, member):
+        # NOTE(froyo): OVN provider allow to create member without param
+        # subnet_id, in that case the driver search it according to the
+        # pool_id, but it is not propagated to Octavia. In this case, if
+        # the member is deleted, Octavia send the object without subnet_id.
+        subnet_id = member.subnet_id
+        if (isinstance(subnet_id, o_datamodels.UnsetType) or not subnet_id):
+            subnet_id, subnet_cidr = self._ovn_helper._get_subnet_from_pool(
+                member.pool_id)
+            if not (subnet_id and
+                    self._ovn_helper._check_ip_in_subnet(member.address,
+                                                         subnet_cidr)):
+                msg = _('Subnet is required, or Loadbalancer associated with '
+                        'Pool must have a subnet, for Member deletion if it is'
+                        'with OVN Provider Driver if it is not the same as '
+                        'LB VIP subnet')
+                raise driver_exceptions.UnsupportedOptionError(
+                    user_fault_string=msg,
+                    operator_fault_string=msg)
+
         request_info = {'id': member.member_id,
                         'address': member.address,
                         'protocol_port': member.protocol_port,
                         'pool_id': member.pool_id,
-                        'subnet_id': member.subnet_id}
+                        'subnet_id': subnet_id}
         request = {'type': ovn_const.REQ_TYPE_MEMBER_DELETE,
                    'info': request_info}
         self._ovn_helper.add_request(request)
@@ -286,13 +312,32 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         request_info = {'id': member.member_id,
                         'address': member.address,
                         'pool_id': member.pool_id,
-                        'subnet_id': member.subnet_id,
+                        'subnet_id': subnet_id,
                         'action': ovn_const.REQ_INFO_MEMBER_DELETED}
         request = {'type': ovn_const.REQ_TYPE_HANDLE_MEMBER_DVR,
                    'info': request_info}
         self._ovn_helper.add_request(request)
 
     def member_update(self, old_member, new_member):
+        # NOTE(froyo): OVN provider allow to create member without param
+        # subnet_id, in that case the driver search it according to the
+        # pool_id, but it is not propagated to Octavia. In this case, if
+        # the member is updated, Octavia send the object without subnet_id.
+        subnet_id = old_member.subnet_id
+        if (isinstance(subnet_id, o_datamodels.UnsetType) or not subnet_id):
+            subnet_id, subnet_cidr = self._ovn_helper._get_subnet_from_pool(
+                old_member.pool_id)
+            if not (subnet_id and
+                    self._ovn_helper._check_ip_in_subnet(new_member.address,
+                                                         subnet_cidr)):
+                msg = _('Subnet is required, or Loadbalancer associated with '
+                        'Pool must have a subnet, for Member update '
+                        'with OVN Provider Driver if it is not the same as '
+                        'LB VIP subnet')
+                raise driver_exceptions.UnsupportedOptionError(
+                    user_fault_string=msg,
+                    operator_fault_string=msg)
+
         # Validate monitoring options if present
         self._check_member_monitor_options(new_member)
         if new_member.address and self._ip_version_differs(new_member):
@@ -301,7 +346,7 @@ class OvnProviderDriver(driver_base.ProviderDriver):
                         'address': old_member.address,
                         'protocol_port': old_member.protocol_port,
                         'pool_id': old_member.pool_id,
-                        'subnet_id': old_member.subnet_id,
+                        'subnet_id': subnet_id,
                         'old_admin_state_up': old_member.admin_state_up}
         if not isinstance(new_member.admin_state_up, o_datamodels.UnsetType):
             request_info['admin_state_up'] = new_member.admin_state_up
@@ -317,19 +362,34 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         pool = external_ids[pool_key]
         existing_members = pool.split(',') if pool else []
         members_to_delete = copy.copy(existing_members)
+        pool_subnet_id = None
         for member in members:
             if (self._check_monitor_options(member) or
                     member.address and self._ip_version_differs(member)):
                 skipped_members.append(member.member_id)
                 continue
-            # NOTE(mjozefcz): We need to have subnet_id information.
-            if (isinstance(member.subnet_id, o_datamodels.UnsetType) or
-                    not member.subnet_id):
-                msg = _('Subnet is required for Member creation '
-                        'with OVN Provider Driver')
-                raise driver_exceptions.UnsupportedOptionError(
-                    user_fault_string=msg,
-                    operator_fault_string=msg)
+            # NOTE(froyo): if subnet_id not provided, lets try to get it
+            # from the member pool_id
+            subnet_id = member.subnet_id
+            if (isinstance(subnet_id, o_datamodels.UnsetType) or
+                    not subnet_id):
+                if not pool_subnet_id:
+                    pool_subnet_id, pool_subnet_cidr = (
+                        self._ovn_helper._get_subnet_from_pool(pool_id))
+                if pool_subnet_id:
+                    if (self._ovn_helper._check_ip_in_subnet(
+                            member.address, pool_subnet_cidr)):
+                        member.subnet_id = pool_subnet_id
+                # NOTE(mjozefcz): We need to have subnet_id information.
+                if not member.subnet_id:
+                    msg = _('Subnet is required, or Loadbalancer associated '
+                            'with Pool must have a subnet, for Member '
+                            'batch update with OVN Provider Driver if it is '
+                            'not the same as LB VIP subnet')
+                    raise driver_exceptions.UnsupportedOptionError(
+                        user_fault_string=msg,
+                        operator_fault_string=msg)
+
             admin_state_up = member.admin_state_up
             if isinstance(admin_state_up, o_datamodels.UnsetType):
                 admin_state_up = True
@@ -355,12 +415,14 @@ class OvnProviderDriver(driver_base.ProviderDriver):
 
         for member in members_to_delete:
             member_info = member.split('_')
+            member_ip, member_port, subnet_id = (
+                self._ovn_helper._extract_member_info(member)[0])
             request_info = {'id': member_info[1],
-                            'address': member_info[2].split(':')[0],
-                            'protocol_port': member_info[2].split(':')[1],
+                            'address': member_ip,
+                            'protocol_port': member_port,
                             'pool_id': pool_id}
             if len(member_info) == 4:
-                request_info['subnet_id'] = member_info[3]
+                request_info['subnet_id'] = subnet_id
             request = {'type': ovn_const.REQ_TYPE_MEMBER_DELETE,
                        'info': request_info}
             request_list.append(request)
