@@ -57,17 +57,26 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         self.fake_neutron_client = mock.MagicMock()
         clients.get_neutron_client = mock.MagicMock()
         clients.get_neutron_client.return_value = self.fake_neutron_client
+        self.fake_neutron_client.show_network = self._mock_show_network
         self.fake_neutron_client.show_subnet = self._mock_show_subnet
         self.fake_neutron_client.list_ports = self._mock_list_ports
         self.fake_neutron_client.show_port = self._mock_show_port
         self.fake_neutron_client.delete_port.return_value = True
         self._local_net_cache = {}
+        self._local_cidr_cache = {}
         self._local_port_cache = {'ports': []}
         self.core_plugin = directory.get_plugin()
 
+    def _mock_show_network(self, network_id):
+        network = {}
+        network['id'] = network_id
+        network['provider:physical_network'] = None
+        return {'network': network}
+
     def _mock_show_subnet(self, subnet_id):
         subnet = {}
         subnet['network_id'] = self._local_net_cache[subnet_id]
+        subnet['cidr'] = self._local_cidr_cache[subnet_id]
         return {'subnet': subnet}
 
     def _mock_list_ports(self, **kwargs):
@@ -180,6 +189,11 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                 external_ids[
                     ovn_const.LB_EXT_IDS_LS_REFS_KEY] = jsonutils.loads(
                         ls_refs)
+            member_status = external_ids.get(ovn_const.OVN_MEMBER_STATUS_KEY)
+            if member_status:
+                external_ids[
+                    ovn_const.OVN_MEMBER_STATUS_KEY] = jsonutils.loads(
+                        member_status)
             lb_dict = {'name': lb.name, 'protocol': lb.protocol,
                        'vips': lb.vips, 'external_ids': external_ids}
             try:
@@ -248,6 +262,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                                   cidr)
         subnet = self.deserialize(self.fmt, res)['subnet']
         self._local_net_cache[subnet['id']] = n1['network']['id']
+        self._local_cidr_cache[subnet['id']] = subnet['cidr']
 
         port = self._make_port(self.fmt, n1['network']['id'])
         if router_id:
@@ -319,6 +334,77 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
         expected_lbs = self._make_expected_lbs(lb_data)
         self._validate_loadbalancers(expected_lbs)
 
+    def _create_load_balancer_custom_lr_ls_and_validate(
+        self, admin_state_up=True, create_router=True,
+            force_retry_ls_to_lr_assoc=True):
+
+        self._o_driver_lib.update_loadbalancer_status.reset_mock()
+        r_id = self._create_router('r1') if create_router else None
+
+        net_info = []
+        net_info.append(self._create_net("n1", "10.0.1.0/24", r_id))
+        net_info.append(self._create_net("n2", "10.0.2.0/24", r_id))
+        net_info.append(self._create_net("n3", "10.0.3.0/24", r_id))
+
+        lb_data = {}
+        lb_data['model'] = self._create_lb_model(
+            vip=net_info[0][2],
+            vip_network_id=net_info[0][0],
+            vip_subnet_id=net_info[0][1],
+            vip_port_id=net_info[0][3],
+            admin_state_up=admin_state_up)
+
+        lb_data[ovn_const.LB_EXT_IDS_LR_REF_KEY] = \
+            (ovn_const.LR_REF_KEY_HEADER + r_id)
+        lb_data['vip_net_info'] = net_info[0]
+        lb_data[ovn_const.LB_EXT_IDS_LS_REFS_KEY] = {}
+        lb_data['listeners'] = []
+        lb_data['pools'] = []
+        self._update_ls_refs(lb_data, net_info[0][0])
+        ls = [ovn_const.LR_REF_KEY_HEADER + net[0] for net in net_info]
+
+        if force_retry_ls_to_lr_assoc:
+            ls_foo = copy.deepcopy(ls)
+            ls_foo.append('neutron-foo')
+            self.ovn_driver._ovn_helper._find_ls_for_lr = mock.MagicMock()
+            self.ovn_driver._ovn_helper._find_ls_for_lr.side_effect = \
+                [ls_foo, ls]
+
+        self.ovn_driver.loadbalancer_create(lb_data['model'])
+
+        name = '%s%s' % (ovn_const.LB_VIP_PORT_PREFIX,
+                         lb_data['model'].loadbalancer_id)
+        self.driver.update_port(
+            self.context, net_info[0][3], {'port': {'name': name}})
+
+        if lb_data['model'].admin_state_up:
+            expected_status = {
+                'loadbalancers': [{"id": lb_data['model'].loadbalancer_id,
+                                   "provisioning_status": "ACTIVE",
+                                   "operating_status": o_constants.ONLINE}]
+            }
+        else:
+            expected_status = {
+                'loadbalancers': [{"id": lb_data['model'].loadbalancer_id,
+                                   "provisioning_status": "ACTIVE",
+                                   "operating_status": o_constants.OFFLINE}]
+            }
+        self._wait_for_status_and_validate(lb_data, [expected_status])
+        self.assertTrue(
+            self._is_lb_associated_to_ls(
+                lb_data['model'].loadbalancer_id,
+                ovn_const.LR_REF_KEY_HEADER + net_info[0][0]))
+
+        # NOTE(froyo): Just to check all net connected to lr have a
+        # reference to lb
+        for net_id in ls:
+            self.assertTrue(
+                self._is_lb_associated_to_ls(
+                    lb_data['model'].loadbalancer_id,
+                    net_id))
+
+        return lb_data
+
     def _create_load_balancer_and_validate(self, lb_info,
                                            admin_state_up=True,
                                            only_model=False,
@@ -503,6 +589,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
 
         pool_info = {}
         for p in lb_data.get('pools', []):
+            member_status = {}
             external_ids = _get_lb_field_by_protocol(
                 p.protocol.lower(),
                 field='external_ids')
@@ -529,11 +616,14 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                                     'neutron-%s' % port['network_id'], 0)
                                 ex['neutron-%s' % port['network_id']] = act + 1
                                 break
+                member_status[m.member_id] = o_constants.NO_MONITOR
             pool_key = 'pool_' + p.pool_id
             if not p.admin_state_up:
                 pool_key += ':D'
             external_ids[pool_key] = p_members
             pool_info[p.pool_id] = p_members
+            if member_status:
+                external_ids[ovn_const.OVN_MEMBER_STATUS_KEY] = member_status
 
         for listener in lb_data['listeners']:
             expected_vips = _get_lb_field_by_protocol(
@@ -737,12 +827,21 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
             if m.address == member_address:
                 return m
 
-    def _update_member_and_validate(self, lb_data, pool_id, member_address):
+    def _update_member_and_validate(self, lb_data, pool_id, member_address,
+                                    remove_subnet_id=False):
         pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
 
         member = self._get_pool_member(pool, member_address)
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
-        self.ovn_driver.member_update(member, member)
+        old_member = copy.deepcopy(member)
+
+        # NOTE(froyo): In order to test update of member without passing the
+        # subnet_id parameter of the member, just to cover the case when a new
+        # member has been created without passing that argument
+        if remove_subnet_id:
+            old_member.subnet_id = None
+
+        self.ovn_driver.member_update(old_member, member)
         expected_status = {
             'pools': [{'id': pool.pool_id,
                        'provisioning_status': 'ACTIVE'}],
@@ -753,7 +852,8 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
             'listeners': []
         }
         if getattr(member, 'admin_state_up', None):
-            expected_status['members'][0]['operating_status'] = "ONLINE"
+            expected_status['members'][0][
+                'operating_status'] = o_constants.NO_MONITOR
         else:
             expected_status['members'][0]['operating_status'] = "OFFLINE"
         self._wait_for_status_and_validate(lb_data, [expected_status])
@@ -795,7 +895,7 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
                                            check_call=False)
 
     def _delete_member_and_validate(self, lb_data, pool_id, network_id,
-                                    member_address):
+                                    member_address, remove_subnet_id=False):
         pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
         member = self._get_pool_member(pool, member_address)
         pool.members.remove(member)
@@ -806,7 +906,15 @@ class TestOvnOctaviaBase(base.TestOVNFunctionalBase,
             pool_status['operating_status'] = o_constants.OFFLINE
 
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
-        self.ovn_driver.member_delete(member)
+
+        # NOTE(froyo): In order to test deletion of member without passing
+        # the subnet_id parameter of the member, just to cover the case when
+        # a new member has been created without passing that argument
+        m_member = copy.deepcopy(member)
+        if remove_subnet_id:
+            m_member.subnet_id = None
+
+        self.ovn_driver.member_delete(m_member)
         expected_status = {
             'pools': [pool_status],
             'members': [{"id": member.member_id,
